@@ -1,7 +1,7 @@
 /*
 * libslack - http://libslack.org/
 *
-* Copyright (C) 1999, 2000 raf <raf@raf.org>
+* Copyright (C) 1999-2002 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * or visit http://www.gnu.org/copyleft/gpl.html
 *
-* 20000902 raf <raf@raf.org>
+* 20020916 raf <raf@raf.org>
 */
 
 #ifndef LIBSLACK_VSSCANF_H
@@ -28,9 +28,9 @@
 
 #include <slack/hdr.h>
 
-__START_DECLS
-int vsscanf __PROTO ((const char *str, const char *format, va_list args));
-__STOP_DECLS
+_begin_decls
+int vsscanf _args ((const char *str, const char *format, va_list args));
+_end_decls
 
 #endif
 
